@@ -76,7 +76,10 @@ os_progname_init (void)
 #ifdef  OS_HAVE_LINUX_PROCFS
     ssize_t rc;
 
-    rc = readlink("/proc/self/exe", os_progname_buf, sizeof(os_progname_buf));
+    memset(os_progname_buf, 0, sizeof(os_progname_buf));
+    rc = readlink("/proc/self/exe", os_progname_buf,
+        sizeof(os_progname_buf) - 1);
+
     if (rc < 0) {
         return;
     }
@@ -141,7 +144,12 @@ os_mkdir (const char *path, mode_t mode)
 
     for (s = strchr(p + 1, '/'); s != NULL; s = strchr(s + 1, '/')) {
         *s = '\0';
-        mkdir(p, mode);
+
+        /* Note, we have nothing to do with errors in creation of
+         * intermediate directories, but status of creation of the
+         * final directory in the path is checked below
+         */
+        (void) mkdir(p, mode);
         *s = '/';
     }
 
