@@ -20,9 +20,11 @@ usage (char **argv)
     printf("    %s [options]\n", argv[0]);
     printf("\n");
     printf("Options are:\n");
-    printf("    -d   enable debug mode\n");
-    printf("    -t   enable protocol trace\n");
-    printf("    -h   print help page\n");
+    printf("    -test-fast  Fast discovery mode, for testing.\n");
+    printf("    -test-auto  automatic protocol selection, for testing\n");
+    printf("    -d          enable debug mode\n");
+    printf("    -t          enable protocol trace\n");
+    printf("    -h          print help page\n");
 
     exit(0);
 }
@@ -67,7 +69,15 @@ main (int argc, char **argv)
 
     /* Parse command-line options */
     for (i = 1; i < argc; i ++) {
-        if (!strcmp(argv[i], "-d")) {
+        if (!strcmp(argv[i], "-test-fast")) {
+            conf.wsdd_mode = WSDD_FAST;
+        } else if (!strcmp(argv[i], "--test-fast")) {
+            conf.wsdd_mode = WSDD_FAST;
+        } else if (!strcmp(argv[i], "-test-auto")) {
+            conf.proto_auto = true;
+        } else if (!strcmp(argv[i], "--test-auto")) {
+            conf.proto_auto = true;
+        } else if (!strcmp(argv[i], "-d")) {
             conf.dbg_enabled = true;
         } else if (!strcmp(argv[i], "-t")) {
             conf.dbg_trace = str_dup("./");
@@ -102,6 +112,8 @@ main (int argc, char **argv)
             printf("  %s = %s, %s\n", devinfo->name,
                 http_uri_str(endpoint->uri), id_proto_name(endpoint->proto));
         }
+
+        zeroconf_devinfo_free(devinfo);
     }
 
     zeroconf_device_list_free(devices);
